@@ -12,8 +12,15 @@
 #' - `run_script(script, parameters)`: Run a script on the VM. For a Linux VM, this will be a shell script; for a Windows VM, a PowerShell script. Pass the script as a character vector.
 #' - `sync_vm_status()`: Check the status of the VM.
 #' - `resize(size, deallocate=FALSE, wait=FALSE)`: Resize the VM. Optionally stop and deallocate it first (may sometimes be necessary).
+#' - `redeploy()`: Redeploy the VM.
+#' - `reimage()`: Reimage the VM.
 #' - `get_public_ip_address(nic=1, config=1)`: Get the public IP address of the VM. Returns NA if the VM is shut down, or is not publicly accessible.
 #' - `get_private_ip_address(nic=1, config=1)`: Get the private IP address of the VM.
+#' - `get_public_ip_resource(nic=1, config=1)`: Get the Azure resource for the VM's public IP address.
+#' - `get_nic(nic=1)`: Get the VM's network interface resource.
+#' - `get_vnet(nic=1, config=1)`: Get the VM's virtual network resource.
+#' - `get_nsg(nic=1, config=1)`: Get the VM's network security group resource. Note that an NSG can be attached to either the VM's network interface or to its virtual network subnet; if there is an NSG attached to both, this method returns a list containing the two NSG resource objects.
+#' - `get_disk(disk="os")`: Get a managed disk resource attached to the VM. The `disk` argument can be "os" for the OS disk, or a number indicating the LUN of a data disk. AzureVM only supports managed disks.
 #' - `add_extension(publisher, type, version, settings=list(), protected_settings=list(), key_vault_settings=list())`: Add an extension to the VM.
 #' - `do_vm_operation(...)`: Carry out an arbitrary operation on the VM resource. See the `do_operation` method of the [AzureRMR::az_resource] class for more details.
 #'
@@ -52,45 +59,14 @@ public=list(
     start=function(wait=TRUE)
     {
         self$do_operation("start", http_verb="POST")
-        # Sys.sleep(2)
-        if(wait)
-        {
-            for(i in 1:100)
-            {
-                Sys.sleep(5)
-                self$sync_vm_status()
-                if(length(self$status) == 2 &&
-                    self$status[1] == "succeeded" &&
-                    self$status[2] == "running")
-                    break
-            }
-            if(length(self$status) < 2 ||
-                self$status[1] != "succeeded" ||
-                self$status[2] != "running")
-                stop("Unable to start VM", call.=FALSE)
-        }
+        if(wait) private$wait_for_success("start")
     },
 
     restart=function(wait=TRUE)
     {
+        status <- self$sync_vm_status()
         self$do_operation("restart", http_verb="POST")
-        # Sys.sleep(2)
-        if(wait)
-        {
-            for(i in 1:100)
-            {
-                Sys.sleep(5)
-                self$sync_vm_status()
-                if(length(self$status) == 2 &&
-                    self$status[1] == "succeeded" &&
-                    self$status[2] == "running")
-                    break
-            }
-            if(length(self$status) < 2 ||
-                self$status[1] != "succeeded" ||
-                self$status[2] != "running")
-                stop("Unable to restart VM", call.=FALSE)
-        }
+        if(wait) private$wait_for_success("restart")
     },
 
     stop=function(deallocate=TRUE, wait=FALSE)
@@ -123,14 +99,16 @@ public=list(
 
         if(wait)
         {
+            size <- tolower(size)
             for(i in 1:100)
             {
                 self$sync_vm_status()
-                if(properties$hardwareProfile$vmSize == size)
+                newsize <- tolower(properties$hardwareProfile$vmSize)
+                if(newsize == size)
                     break
                 Sys.sleep(5)
             }
-            if(properties$hardwareProfile$vmSize != size)
+            if(newsize != size)
                 stop("Unable to resize VM", call.=FALSE)
         }
     },
@@ -155,7 +133,7 @@ public=list(
 
     get_public_ip_address=function(nic=1, config=1)
     {
-        ip <- private$get_ip(nic, config)
+        ip <- self$get_public_ip_resource(nic, config)
         if(is.null(ip) || is.null(ip$properties$ipAddress))
             return(NA_character_)
 
@@ -164,10 +142,84 @@ public=list(
 
     get_private_ip_address=function(nic=1, config=1)
     {
-        nic <- private$get_nic(nic)
+        nic <- self$get_nic(nic)
         nic$properties$ipConfigurations[[config]]$properties$privateIPAddress
     },
 
+    get_public_ip_resource=function(nic=1, config=1)
+    {
+        nic <- self$get_nic(nic)
+        ip_id <- nic$properties$ipConfigurations[[config]]$properties$publicIPAddress$id
+        if(is_empty(ip_id))
+            return(NULL)
+        az_resource$new(self$token, self$subscription, id=ip_id, api_version=self$ip_api_version)
+    },
+
+    get_nic=function(nic=1)
+    {
+        nic_id <- self$properties$networkProfile$networkInterfaces[[nic]]$id
+        if(is_empty(nic_id))
+            stop("Network interface resource not found", call.=FALSE)
+        az_resource$new(self$token, self$subscription, id=nic_id, api_version=self$nic_api_version)
+    },
+
+    get_vnet=function(nic=1, config=1)
+    {
+        nic <- self$get_nic(nic)
+        subnet_id <- nic$properties$ipConfigurations[[config]]$properties$subnet$id
+        vnet_id <- sub("/subnets/[^/]+$", "", subnet_id)
+        az_resource$new(self$token, self$subscription, id=vnet_id)
+    },
+
+    get_nsg=function(nic=1, config=1)
+    {
+        vnet <- self$get_vnet(nic, config)
+        nic <- self$get_nic(nic)
+
+        nic_nsg_id <- nic$properties$networkSecurityGroup$id
+        nic_nsg <- if(!is.null(nic_nsg_id))
+            az_resource$new(self$token, self$subscription, id=nic_nsg_id)
+        else NULL
+
+        # go through list of subnets, find the one where this VM is located
+        found <- FALSE
+        nic_id <- tolower(nic$id)
+        for(sn in vnet$properties$subnets)
+        {
+            nics <- tolower(unlist(sn$properties$ipConfigurations))
+            if(any(grepl(nic_id, nics, fixed=TRUE)))
+            {
+                found <- TRUE
+                break
+            }
+        }
+        if(!found)
+            stop("Error locating subnet for this network configuration", call.=FALSE)
+
+        subnet_nsg_id <- sn$properties$networkSecurityGroup$id
+        subnet_nsg <- if(!is.null(subnet_nsg_id))
+            az_resource$new(self$token, self$subscription, id=subnet_nsg_id)
+        else NULL
+
+        if(is.null(nic_nsg) && is.null(subnet_nsg))
+            NULL
+        else if(is.null(nic_nsg) && !is.null(subnet_nsg))
+            subnet_nsg
+        else if(!is.null(nic_nsg) && is.null(subnet_nsg))
+            nic_nsg
+        else(list(nic_nsg, subnet_nsg))
+    },
+
+    get_disk=function(disk="os")
+    {
+        id <- if(disk == "os")
+            self$properties$storageProfile$osDisk$managedDisk$id
+        else if(is.numeric(disk))
+            self$properties$storageProfile$dataDisks[[disk]]$managedDisk$id
+        else stop("Invalid disk argument: should be 'os', or the data disk number", call.=FALSE)
+        az_resource$new(self$token, self$subscription, id=id)
+    },
+
     add_extension=function(publisher, type, version, settings=list(),
         protected_settings=list(), key_vault_settings=list())
     {
@@ -189,6 +241,18 @@ public=list(
         self$do_operation(op, body=list(properties=props), http_verb="PUT")
     },
 
+    redeploy=function()
+    {
+        self$do_operation("redeploy", http_verb="POST")
+        message("Redeployment started. Call the sync_vm_status() method to check progress.")
+    },
+
+    reimage=function()
+    {
+        self$do_operation("reimage", http_verb="POST")
+        message("Reimage started. Call the sync_vm_status() method to check progress.")
+    },
+
     print=function(...)
     {
         cat("<Azure virtual machine resource ", self$name, ">\n", sep="")
@@ -213,26 +277,26 @@ public=list(
 
 private=list(
 
-    get_nic=function(nic=1)
-    {
-        nic_id <- self$properties$networkProfile$networkInterfaces[[nic]]$id
-        if(is_empty(nic_id))
-            stop("Network interface resource not found", call.=FALSE)
-        az_resource$new(self$token, self$subscription, id=nic_id, api_version=self$nic_api_version)
-    },
-
-    get_ip=function(nic=1, config=1)
-    {
-        nic <- private$get_nic(nic)
-        ip_id <- nic$properties$ipConfigurations[[config]]$properties$publicIPAddress$id
-        if(is_empty(ip_id))
-            return(NULL)
-        az_resource$new(self$token, self$subscription, id=ip_id, api_version=self$ip_api_version)
-    },
-
     init_and_deploy=function(...)
     {
         stop("Do not use 'az_vm_resource' to create a new VM", call.=FALSE)
+    },
+
+    wait_for_success=function(op)
+    {
+        for(i in 1:1000)
+        {
+            Sys.sleep(5)
+            self$sync_vm_status()
+            if(length(self$status) == 2 &&
+                self$status[1] == "succeeded" &&
+                self$status[2] == "running")
+                break
+        }
+        if(length(self$status) < 2 ||
+            self$status[1] != "succeeded" ||
+            self$status[2] != "running")
+            stop("Unable to ", op, " VM", call.=FALSE)
     }
 ))
 
